package com.usky.system.controller.web;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.log.annotation.Log;
import com.usky.common.log.enums.BusinessType;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.system.controller.web.page.TableDataInfo;
import com.usky.system.domain.SysCheckCode;
import com.usky.system.domain.SysOperLog;
import com.usky.system.mapper.SysCheckCodeMapper;
import com.usky.system.service.ISysOperLogService;
import com.usky.system.service.util.Operation;
import com.usky.system.service.vo.SysOperLogExportVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;

/**
 * 操作日志记录
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/operlog")
public class SysOperlogController extends BaseController {
    @Autowired
    private ISysOperLogService operLogService;

    @Autowired
    private SysCheckCodeMapper sysCheckCodeMapper;

    Operation operation = new Operation();

    @GetMapping("/list")
    public ApiResult<TableDataInfo> list(SysOperLog operLog) {
        startPage();
        List<SysOperLog> list = operLogService.selectOperLogList(operLog);
        for (int i = 0; i < list.size(); i++) {
            LambdaQueryWrapper<SysCheckCode> query = new LambdaQueryWrapper<>();
            query.select(SysCheckCode::getCheckCode)
                    .eq(SysCheckCode::getPlaintext,list.get(i).getOperId())
                    .eq(SysCheckCode::getCheckCodeType,4);
            SysCheckCode sysCheckCode = sysCheckCodeMapper.selectOne(query);
            if (sysCheckCode==null){
                throw new BusinessException("未查询到数据！");
            }
            String checkCode1 = sysCheckCode.getCheckCode();
            try {
                String checkCode = operation.CBCMAC((list.get(i).getTitle() + list.get(i).getBusinessType().toString() + list.get(i).getMethod() +
                        list.get(i).getOperName() + list.get(i).getOperUrl() + list.get(i).getOperIp() + list.get(i).getOperTime().toString() + list.get(i).getTenantId().toString()).getBytes("UTF-8"));
                if (!checkCode.equals(checkCode1)){
                    throw new BusinessException("【完整性】校验失败，日志数据是被破坏或者篡改的");
                }
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
        }
        return ApiResult.success(getDataTable(list));
    }

    /**
     * 操作日志-导出
     *
     * @param response
     * @param operLog
     * @throws IOException
     */
    @PostMapping("listExport")
    public void export(HttpServletResponse response,
                       @RequestParam(value = "exportTitle") String exportTitle,
                       SysOperLog operLog) throws IOException {
        List<SysOperLogExportVO> list = operLogService.selectOperLogListExport(operLog);
        ExcelUtil<SysOperLogExportVO> util = new ExcelUtil<SysOperLogExportVO>(SysOperLogExportVO.class);
        util.exportExcel(response, list, exportTitle, exportTitle);
    }


    @DeleteMapping("/{operIds}")
    @Log(title = "操作日志", businessType = BusinessType.DELETE)
    public ApiResult<Void> remove(@PathVariable Long[] operIds) {
        return toAjax(operLogService.deleteOperLogByIds(operIds));
    }


    @DeleteMapping("/clean")
    @Log(title = "操作日志-清空", businessType = BusinessType.DELETE)
    public ApiResult<Void> clean() {
        operLogService.cleanOperLog();
        return ApiResult.success();
    }

    @PostMapping
    public ApiResult<Void> add(@RequestBody SysOperLog operLog) {
        operLogService.insertOperlog(operLog);
        return ApiResult.success();
    }
}
